@@ -36,11 +36,13 @@
 #'   `size_factors` is either a numeric vector with positive entries that has the same lengths as columns in the data
 #'   that specifies the size factors that are used.
 #'   Or it can be a string that species the method that is used to estimate the size factors
-#'   (one of \code{c("normed_sum", "deconvolution", "poscounts")}).
+#'   (one of \code{c("normed_sum", "deconvolution", "poscounts", "ratio")}).
 #'   Note that \code{"normed_sum"} and \code{"poscounts"} are fairly
-#'   simple methods and can lead to suboptimal results. For the best performance, I recommend to use
+#'   simple methods and can lead to suboptimal results. For the best performance on data with many zeros, I recommend to use
 #'   `size_factors = "deconvolution"` which calls `scran::calculateSumFactors()`. However, you need
-#'   to separately install the `scran` package from Bioconductor for this method to work.
+#'   to separately install the `scran` package from Bioconductor for this method to work. For small datasets
+#'   common for bulk RNA-seq experiments, I recommend to use `size_factors = "ratio"`, which uses the same
+#'   procedure as DESeq2 and edgeR.
 #'   Also note that `size_factors = 1` and `size_factors = FALSE` are equivalent. If only a single gene is given,
 #'   no size factor is estimated (ie. `size_factors = 1`). Default: `"normed_sum"`.
 #' @param overdispersion the simplest count model is the Poisson model. However, the Poisson model
@@ -82,6 +84,8 @@
 #'   Default: `FALSE` which means that the data is not subsampled. If set to `TRUE`, at most 1,000 samples
 #'   are considered. Otherwise the parameter just specifies the number of samples that are considered
 #'   for each gene to estimate the overdispersion.
+#' @param use_assay Specify which assay to use. Default: `NULL`, which means that if available 'counts'
+#'   are used. Otherwise an error is thrown except if there is only a single assay.
 #' @param on_disk a boolean that indicates if the dataset is loaded into memory or if it is kept on disk
 #'   to reduce the memory usage. Processing in memory can be significantly faster than on disk.
 #'   Default: `NULL` which means that the data is only processed in memory if `data` is an in-memory
@@ -100,7 +104,8 @@
 #'    equation (5) is used. To handle the large number of zeros the geometric means are calculated for
 #'    \eqn{Y + 0.5} and ignored during the calculation of the median. Columns with all zeros get a
 #'    default size factor of \eqn{0.001}. If `size_factors = "deconvolution"`, the method
-#'    `scran::calculateSumFactors()` is called.
+#'    `scran::calculateSumFactors()` is called. If `size_factors = "ratio"`, the unmodified procedure
+#'    from Anders and Huber (2010) in equation (5) is used.
 #' 2. The dispersion estimates are initialized based on the moments of each row of \eqn{Y}.
 #' 3. The coefficients of the model are estimated.\cr
 #'    If all samples belong to the same condition (i.e. `design = ~ 1`), the betas are estimated using
@@ -216,13 +221,14 @@ glm_gp <- function(data,
                    col_data = NULL,
                    reference_level = NULL,
                    offset = 0,
-                   size_factors = c("normed_sum", "deconvolution", "poscounts"),
+                   size_factors = c("normed_sum", "deconvolution", "poscounts", "ratio"),
                    overdispersion = TRUE,
                    overdispersion_shrinkage = TRUE,
                    ridge_penalty = 0,
                    do_cox_reid_adjustment = TRUE,
                    subsample = FALSE,
                    on_disk = NULL,
+                   use_assay = NULL,
                    verbose = FALSE){
 
   # Validate `data`
@@ -238,7 +244,7 @@ glm_gp <- function(data,
   if(is.vector(data)){
     data <- matrix(data, nrow = 1)
   }
-  data_mat <- handle_data_parameter(data, on_disk)
+  data_mat <- handle_data_parameter(data, use_assay, on_disk ,verbose = verbose)
 
   # Convert the formula to a model_matrix
   col_data <- get_col_data(data, col_data)
@@ -283,7 +289,7 @@ glm_gp <- function(data,
 }
 
 
-handle_data_parameter <- function(data, on_disk){
+handle_data_parameter <- function(data, use_assay, on_disk, verbose = FALSE){
   if(is.matrix(data)){
     if(! is.numeric(data)){
       stop("The data argument must consist of numeric values and not of ", mode(data), " values")
@@ -304,10 +310,38 @@ handle_data_parameter <- function(data, on_disk){
       stop("Illegal argument type for on_disk. Can only handle 'NULL', 'TRUE', or 'FALSE'")
     }
   }else if(is(data, "SummarizedExperiment")){
-    data_mat <- handle_data_parameter(SummarizedExperiment::assay(data), on_disk)
+    if(is.null(use_assay)){
+      use_assay <- if("counts" %in% SummarizedExperiment::assayNames(data)) "counts" else {
+        assay_names <-  SummarizedExperiment::assayNames(data)
+        if(is.null(assay_names)){
+          if(verbose) message("Using unnamed assay")
+          1
+        }else if(length(assay_names) == 1){
+          if(verbose) message("Using assay '", assay_names, "'")
+          1
+        }else{
+          stop("Please specify the assay name.")
+        }
+      }
+    }
+    data_mat <- handle_data_parameter(SummarizedExperiment::assay(data, use_assay), use_assay = NULL, on_disk)
   }else if(canCoerce(data, "SummarizedExperiment")){
     se <- as(data, "SummarizedExperiment")
-    data_mat <- handle_data_parameter(SummarizedExperiment::assay(se), on_disk)
+    if(is.null(use_assay)){
+      use_assay <- if("counts" %in% SummarizedExperiment::assayNames(data)) "counts" else {
+        assay_names <-  SummarizedExperiment::assayNames(data)
+        if(is.null(assay_names)){
+          if(verbose) message("Using unnamed assay")
+          1
+        }else if(length(assay_names) == 1){
+          if(verbose) message("Using assay '", assay_names, "'")
+          1
+        }else{
+          stop("Please specify the assay name.")
+        }
+      }
+    }
+    data_mat <- handle_data_parameter(SummarizedExperiment::assay(se, use_assay), use_assay = NULL, on_disk)
   }else if(is(data, "dgCMatrix") || is(data, "dgTMatrix")) {
     if(isTRUE(on_disk)){
       data_mat <- HDF5Array::writeHDF5Array(data)
@@ -404,8 +438,7 @@ handle_design_parameter <- function(design, data, col_data, reference_level){
     stop("The model_matrix has more columns (", ncol(model_matrix),
          ") than the there are samples in the data matrix (", n_samples, " columns).\n",
          "Too few replicates / too many coefficients to fit model.\n",
-         "The head of the design matrix: \n", format_matrix(head(model_matrix, n = 3)), "\n",
-         "The head of the data: \n", format_matrix(head(data[,seq_len(min(5, ncol(data))),drop=FALSE], n = 3)))
+         "The head of the design matrix: \n", format_matrix(head(model_matrix, n = 3)))
   }
 
   # Check rank of model_matrix
@@ -426,6 +459,9 @@ handle_design_parameter <- function(design, data, col_data, reference_level){
   rownames(model_matrix) <- colnames(data)
   validate_model_matrix(model_matrix, data)
   model_matrix <- add_attr_if_intercept(model_matrix)
+  if(ignore_degeneracy){
+    attr(model_matrix, "ignore_degeneracy") <- TRUE
+  }
   list(model_matrix = model_matrix, design_formula = design_formula)
 }
 
