@@ -8,7 +8,9 @@
 #' @param fit object of class `glmGamPoi`. Usually the result of calling `glm_gp(data, ...)`
 #' @param contrast The contrast to test. Can be a single column name (quoted or as a string)
 #'   that is removed from the  full model matrix of `fit`. Or a complex contrast comparing two
-#'   or more columns: e.g. `A - B`, `"A - 3 * B"`, `(A + B) / 2 - C` etc. \cr
+#'   or more columns: e.g. `A - B`, `"A - 3 * B"`, `(A + B) / 2 - C` etc. For complicated
+#'   experimental design that involved nested conditions, you specify the condition level to compare
+#'   using the `cond()` helper function. \cr
 #'   Only one of `contrast` or `reduced_design` must be specified.
 #' @param reduced_design a specification of the reduced design used as a comparison to see what
 #'   how much better `fit` describes the data.
@@ -23,7 +25,8 @@
 #'   (e.g. `subset_to = cell_type == "T-cells"`). Note that if this argument is set a new
 #'   the model for the `full_design` is re-fit.\cr
 #'   Default: `NULL` which means that the data is not subset.
-#' @param pseudobulk_by a vector with the same length as `ncol(fit$data)` that is used to
+#' @param pseudobulk_by *DEPRECTATED*, please use the [pseudobulk] function instead. \cr
+#'   A vector with the same length as `ncol(fit$data)` that is used to
 #'   split the columns into different groups (calls [split()]). `pseudobulk_by` can also be an
 #'   expression that evaluates to a vector. The expression can reference columns from `colData(fit$data)`. \cr
 #'   The counts are summed across the groups
@@ -40,6 +43,22 @@
 #'   order. Default: `FALSE`.
 #' @param n_max the maximum number of rows to return. Default: `Inf` which means that all
 #'   rows are returned
+#' @param max_lfc set the maximum absolute log fold change that is returned. Large log fold changes
+#'   occur for lowly expressed genes because the ratio of two small numbers can be impractically large. For example, limiting
+#'   the range of log fold changes can clarify the patterns in a volcano plot. Default: `10` which
+#'   corresponds to a thousand-fold (2^10) increase in expression.
+#'
+#' @details
+#' The `cond()` helper function simplifies the specification of a contrast for complex experimental designs.
+#' Instead of working out which combination of coefficients corresponds to a research question,
+#' you can simply specify the two conditions that you want to compare.
+#'
+#' You can only call the `cond` function inside the `contrast` argument. The arguments are the selected factor levels
+#' for each covariate. To compare two conditions, simply subtract the two `cond` calls. Internally, the package
+#' calls [model.matrix] using the provided values and the original formula from the fit to produce a vector.
+#' Subtracting two of these vectors produces a contrast vector. Missing covariates are filled with the first factor level
+#' or zero for numerical covariates.
+#'
 #'
 #' @return a `data.frame` with the following columns
 #' \describe{
@@ -54,62 +73,51 @@
 #' }
 #'
 #' @examples
-#'   Y <- matrix(rnbinom(n = 30 * 100, mu = 4, size = 0.3), nrow = 30, ncol = 100)
-#'   annot <- data.frame(sample = sample(LETTERS[1:6], size = 100, replace = TRUE),
-#'                       cont1 = rnorm(100), cont2 = rnorm(100, mean = 30))
-#'   annot$condition <- ifelse(annot$sample %in% c("A", "B", "C"), "ctrl", "treated")
-#'   head(annot)
-#'   se <- SummarizedExperiment::SummarizedExperiment(Y, colData = annot)
-#'   fit <- glm_gp(se, design = ~ condition + cont1 + cont2)
+#'  # Make Data
+#'  Y <- matrix(rnbinom(n = 30 * 100, mu = 4, size = 0.3), nrow = 30, ncol = 100)
+#'  annot <- data.frame(mouse = sample(LETTERS[1:6], size = 100, replace = TRUE),
+#'         celltype = sample(c("Tcell", "Bcell", "Macrophages"), size = 100, replace = TRUE),
+#'         cont1 = rnorm(100), cont2 = rnorm(100, mean = 30))
+#'  annot$condition <- ifelse(annot$mouse %in% c("A", "B", "C"), "ctrl", "treated")
+#'  head(annot)
+#'  se <- SummarizedExperiment::SummarizedExperiment(Y, colData = annot)
 #'
-#'   # Test with reduced design
-#'   res <- test_de(fit, reduced_design = ~ condition + cont1)
-#'   head(res)
-#'
-#'   # Test with contrast argument, the results are identical
-#'   res2 <- test_de(fit, contrast = cont2)
-#'   head(res2)
-#'
-#'   # The column names of fit$Beta are valid variables in the contrast argument
-#'   colnames(fit$Beta)
+#'  # Fit model
+#'  fit <- glm_gp(se, design = ~ condition + celltype + cont1 + cont2)
+#'  # Test with reduced design
+#'  res <- test_de(fit, reduced_design = ~ celltype + cont1 + cont2)
+#'  head(res)
+#'  # Test with contrast argument, the results are identical
+#'  res2 <- test_de(fit, contrast = conditiontreated)
+#'  head(res2)
+#'  # Test with explicit specification of the conditions
+#'  # The results are still identical
+#'  res3 <- test_de(fit, contrast = cond(condition = "treated", celltype = "Bcell") -
+#'                                     cond(condition = "ctrl", celltype = "Bcell"))
+#'  head(res3)
 #'
 #'
-#'   # You can also have more complex contrasts:
-#'   # the following compares cont1 vs cont2:
-#'   test_de(fit, cont1 - cont2, n_max = 4)
-#'
-#'
-#'   # You can also sort the output
-#'   test_de(fit, cont1 - cont2, n_max = 4,
-#'           sort_by = "pval")
-#'
-#'   test_de(fit, cont1 - cont2, n_max = 4,
-#'           sort_by = - abs(f_statistic))
-#'
-#'   # If the data has multiple samples, it is a good
-#'   # idea to aggregate the cell counts by samples.
-#'   # This is called "pseudobulk".
-#'   test_de(fit, contrast = "conditiontreated", n_max = 4,
-#'           pseudobulk_by = sample)
-#'
-#'
-#'   # You can also do the pseudobulk only on a subset of cells:
-#'   cell_types <- sample(c("Tcell", "Bcell", "Makrophages"), size = 100, replace = TRUE)
-#'   test_de(fit, contrast = "conditiontreated", n_max = 4,
-#'           pseudobulk_by = sample,
-#'           subset_to = cell_types == "Bcell")
-#'
-#'
-#'   # Be care full, if you included the cell type information in
-#'   # the original fit, after subsetting the design matrix would
-#'   # be degenerate. To fix this, specify the full_design in 'test_de()'
-#'   SummarizedExperiment::colData(se)$ct <- cell_types
-#'   fit_with_celltype <- glm_gp(se, design = ~ condition + cont1 + cont2 + ct)
-#'   test_de(fit_with_celltype, contrast = cont1, n_max = 4,
-#'           full_design =  ~ condition + cont1 + cont2,
-#'           pseudobulk_by = sample,
-#'           subset_to = ct == "Bcell")
+#'  # The column names of fit$Beta are valid variables in the contrast argument
+#'  colnames(fit$Beta)
+#'  # You can also have more complex contrasts:
+#'  # the following compares cont1 vs cont2:
+#'  test_de(fit, cont1 - cont2, n_max = 4)
+#'  # You can also sort the output
+#'  test_de(fit, cont1 - cont2, n_max = 4,
+#'          sort_by = "pval")
+#'  test_de(fit, cont1 - cont2, n_max = 4,
+#'          sort_by = - abs(f_statistic))
 #'
+#'  # If the data has multiple samples, it is a good
+#'  # idea to aggregate the cell counts by samples.
+#'  # This is called forming a "pseudobulk".
+#'  se_reduced <- pseudobulk(se, group_by = vars(mouse, condition, celltype),
+#'                           cont1 = mean(cont1), cont2 = min(cont2))
+#'  fit_reduced <- glm_gp(se_reduced, design = ~ condition + celltype)
+#'  test_de(fit_reduced, contrast = "conditiontreated", n_max = 4)
+#'  test_de(fit_reduced, contrast = cond(condition = "treated", celltype = "Macrophages") -
+#'                                       cond(condition = "ctrl", celltype = "Macrophages"),
+#'          n_max = 4)
 #'
 #'
 #' @references
@@ -128,17 +136,16 @@ test_de <- function(fit,
                     subset_to = NULL, pseudobulk_by = NULL,
                     pval_adjust_method = "BH", sort_by = NULL,
                     decreasing = FALSE, n_max = Inf,
+                    max_lfc = 10,
                     verbose = FALSE){
   # Capture all NSE variables
-  contrast_capture <- substitute(contrast)
   subset_to_capture <- substitute(subset_to)
   pseudobulk_by_capture <- substitute(pseudobulk_by)
   sort_by_capture <- substitute(sort_by)
-
-  test_de_q(fit, contrast = contrast_capture, reduced_design = reduced_design, full_design = full_design,
+  test_de_q(fit, contrast = {{contrast}}, reduced_design = reduced_design, full_design = full_design,
             subset_to = subset_to_capture, pseudobulk_by = pseudobulk_by_capture,
             pval_adjust_method = pval_adjust_method, sort_by = sort_by_capture,
-            decreasing = decreasing, n_max = n_max,
+            decreasing = decreasing, n_max = n_max, max_lfc = max_lfc,
             verbose = verbose,
             env = parent.frame())
 }
@@ -152,7 +159,7 @@ test_de_q <- function(fit,
                       full_design = fit$model_matrix,
                       subset_to = NULL, pseudobulk_by = NULL,
                       pval_adjust_method = "BH", sort_by = NULL,
-                      decreasing = FALSE, n_max = Inf,
+                      decreasing = FALSE, n_max = Inf, max_lfc = 10,
                       verbose = FALSE,
                       env = parent.frame()){
 
@@ -161,7 +168,7 @@ test_de_q <- function(fit,
     full_design <- handle_design_parameter(full_design, fit$data, SummarizedExperiment::colData(fit$data), NULL)$model_matrix
     ridge_penalty <- NULL
   }
-  if(is.null(reduced_design) == missing(contrast)){
+  if(is.null(reduced_design) == rlang::quo_is_missing(rlang::enquo(contrast))){
     stop("Please provide either an alternative design (formula or matrix) or a contrast ",
          "(name of a column in fit$model_matrix or a combination of them).")
   }
@@ -175,7 +182,7 @@ test_de_q <- function(fit,
     # then calls test_de() with the reduced dataset
     return(test_pseudobulk_q(fit$data, design = full_design,
                              aggregate_cells_by = pseudobulk_by,
-                             contrast = contrast,
+                             contrast = {{contrast}},
                              reduced_design = reduced_design,
                              ridge_penalty = ridge_penalty,
                              subset_to = subset_to_e,
@@ -195,10 +202,10 @@ test_de_q <- function(fit,
 
     fit_subset <- glm_gp(data_subset, design = model_matrix_subset, size_factors = size_factor_subset,
                   overdispersion = fit$overdispersions, on_disk = is_on_disk.glmGamPoi(fit), verbose = verbose)
-    test_res <- test_de_q(fit_subset, contrast = contrast, reduced_design = reduced_design,
+    test_res <- test_de_q(fit_subset, contrast = {{contrast}}, reduced_design = reduced_design,
                           subset_to = NULL, pseudobulk_by = NULL,
                           pval_adjust_method = pval_adjust_method, sort_by = sort_by,
-                          decreasing = decreasing, n_max = n_max,
+                          decreasing = decreasing, n_max = n_max, max_lfc = max_lfc,
                           verbose = verbose, env = env)
     return(test_res)
   }
@@ -207,10 +214,12 @@ test_de_q <- function(fit,
          "'overdispersion_shrinkage = TRUE'.")
   }
   disp_trend <- fit$overdispersion_shrinkage_list$dispersion_trend
-  if(! missing(contrast)){
+  if(!rlang::quo_is_missing(rlang::enquo(contrast))){
     # e.g. a vector with c(1, 0, -1, 0) for contrast = A - C
-    cntrst <- parse_contrast_q(contrast, levels = colnames(fit$model_matrix),
-                               env = env)
+    cntrst <- parse_contrast({{contrast}}, coefficient_names = colnames(fit$model_matrix), formula = fit$design_formula)
+    if(all(abs(cntrst) < 1e-12)){
+      stop("All elements of the contrast vector are zero.")
+    }
     cntrst <- as.matrix(cntrst)
     if(nrow(cntrst) != ncol(fit$model_matrix)){
       stop("The length of the contrast vector does not match the number of coefficients in the model (",
@@ -234,6 +243,8 @@ test_de_q <- function(fit,
       diag(ridge_penalty, nrow = length(ridge_penalty)) %*% rot
     }
     lfc <- fit$Beta %*% cntrst / log(2)
+    lfc[lfc < -max_lfc] <- -max_lfc
+    lfc[lfc > max_lfc] <- max_lfc
   }else{
     # Convert the formula to model matrix
     reduced_design <- handle_design_parameter(reduced_design,  fit$data,
@@ -264,6 +275,9 @@ test_de_q <- function(fit,
   if(verbose){message("Fit reduced model")}
   data <- fit$data
   do_on_disk <- is_on_disk.glmGamPoi(fit)
+  if(isTRUE(attr(fit$model_matrix, "ignore_degeneracy"))){
+    attr(reduced_design, "ignore_degeneracy") <- TRUE
+  }
   fit_alt <- glm_gp(data, design = reduced_design,
                     size_factors = FALSE, # size factors are already in offset
                     offset = fit$Offset,
