@@ -504,8 +504,40 @@ test_that("NA's produced by fitBeta_fisher_scoring don't cause problems", {
 
 
 
+test_that("glm_gp can handle full offset matrices correctly", {
 
 
+  data <- data.frame(fav_food = sample(c("apple", "banana", "cherry"), size = 50, replace = TRUE),
+                     city = sample(c("heidelberg", "paris", "new york"), size = 50, replace = TRUE),
+                     age = rnorm(n = 50, mean = 40, sd = 15))
+  Y <- matrix(rnbinom(n = 10 * 50, mu = 3, size = 1/3.1), nrow = 10, ncol = 50)
+  rownames(Y) <- paste0("gene_", seq_len(10))
+  colnames(Y) <- paste0("person_", seq_len(50))
+
+  sf <- rnorm(n = 50, mean = 1, sd = 0.1)
+  mult <- rlnorm(n = 10, meanlog = log(10), sdlog = 1)
+  Offset <- log(matrix(mult, ncol = 1) %*% matrix(sf, nrow = 1))
+
+  se <- SummarizedExperiment::SummarizedExperiment(Y, colData = data)
+  fit <- glm_gp(se, design = ~ fav_food + city + age, size_factors = sf)
+  fit_offset <- glm_gp(se, design = ~ fav_food + city + age, offset = Offset, size_factors = FALSE)
+
+  expect_equal(fit$Beta[,-1], fit_offset$Beta[,-1], tolerance = 1e-3)
+  expect_equal(fit$Beta[,1], fit_offset$Beta[,1] + log(mult), tolerance = 1e-3)
+})
+
+
+
+test_that("glm_gp can handle gigantic counts", {
+  skip_on_os("windows", "test fails on windows")
+  # Integers with values larger than 2^31-1 = 2,147,483,647 = 2.1 * 10^9 cannot be represented by ints
+  mat <- matrix(rpois(n = 5 * 100, lambda = 3 * 10^9), nrow = 5, ncol = 100)
+  expect_true(any(mat > .Machine$integer.max))
+  rand <- rnorm(100)
+  fit <- glmGamPoi::glm_gp(mat, design = ~ rand)
+  expect_equal(exp(fit$Beta[,1]), rep(3e9, times = 5), tolerance = 0.1)
+  expect_equal(fit$Beta[,2], rep(0, times = 5), tolerance = 1e-4)
+})
 
 
 
