@@ -22,7 +22,23 @@ test_that("test_de works", {
 
   res5 <- test_de(fit, contrast = "-groupB")
   expect_equal(res4, res5, tolerance = 0.1)
+})
 
+test_that("test_de works with 'fact' specifications", {
+  set.seed(1)
+  Y <- matrix(rnbinom(n = 30 * 10, mu = 4, size = 0.3), nrow = 30, ncol  =10)
+  annot <- data.frame(group = sample(c("A", "B"), size = 10, replace = TRUE),
+                      cont1 = rnorm(10), cont2 = rnorm(10, mean = 30))
+  design <- model.matrix(~ group + cont1 + cont2, data = annot)
+  reduced_des <- model.matrix(~ group + cont1, data = annot)
+  fit <- glm_gp(Y, design = design)
+  # Doesn't work because design is a matrix
+  expect_error(test_de(fit, cond(group = "A") - cond(group = "B")))
+
+  fit <- glm_gp(Y, design = ~ group + cont1 + cont2, col_data = annot)
+  res1 <- test_de(fit, cond(group = "B") - cond(group = "A"))
+  res2 <- test_de(fit, groupB)
+  expect_equal(res1, res2)
 })
 
 
@@ -120,7 +136,7 @@ test_that("Pseudo bulk produces same results as making it manually", {
     matrixStats::rowSums2(assay(se), cols = idx)
   }))
   pseudo_design_mat <- do.call(rbind, lapply(splitter, function(idx){
-    matrixStats::colMeans2(fit$model_matrix, rows = idx)
+    matrixStats::colMeans2(fit$model_matrix, rows = idx, useNames = FALSE)
   }))
 
 
@@ -265,3 +281,17 @@ test_that("ridge_penalty is adapted if reduced_design is specified", {
 })
 
 
+test_that("ignore_degeneracy is propagated", {
+  y <- rnbinom(n = 30, mu = 7, size = 1/2.2)
+  X <- cbind(matrix(rnorm(n = 30 * 3), nrow = 30, ncol = 3), 0)
+  Xmod <- X
+  attr(Xmod, "ignore_degeneracy") <- TRUE
+
+  expect_error(glm_gp(y, design = X, ridge_penalty = 0.1))
+
+  # These succeed
+  fit <- glm_gp(y, design = Xmod, ridge_penalty = 0.1)
+  test_de(fit, contrast = c(0, 1, 0, 0))
+})
+
+
